class Namespace:
    def __init__(self, *args):
        self.args = list(args)

    def __call__(self, var):
        all_args = self.args + [var]
        return variable(*all_args)


def arrow_function(value):
    return dict(arrow=value)


def variable(*args):
    return dict(variable=".".join(list(args)))


import dash_html_components as html

from dash.exceptions import PreventUpdate
from .enrich import Output, Input
from .Burger import Burger

URL_ID = "url"
CONTENT_ID = "content"


class Page:
    def __init__(self, id, label, module=None):
        self.id = id
        self.label = label
        self.module = module


class PageCollection:
    def __init__(self, pages, default_page=None, is_authorized=None, unauthorized=None):
        self.pages = pages
        self.is_authorized = is_authorized
        self.unauthorized = html.Div("Unauthorized.") if unauthorized is None else unauthorized
        self.default_page = default_page if default_page is not None else pages[0]

    def navigate_to(self, path):
        if path is None:
            raise PreventUpdate
        # Locate the page. Maybe make this more advanced later, for now just snap the id.
        page_ids = [page.id for page in self.pages]
        next_page = self.default_page
        if path is not None:
            page_id = path[1:]
            if page_id in page_ids:
                next_page = page_id
        # Check that the user is authorized.
        if self.is_authorized is not None:
            if not self.is_authorized(next_page):
                return self.unauthorized
        # At this point, the user is authorized, so the page can safely be rendered.
        return self.pages[page_ids.index(next_page)].module.layout()

    def register_navigation(self, app, content_id=CONTENT_ID, url_id=URL_ID):
        @app.callback(Output(content_id, "children"), [Input(url_id, "pathname")])
        def navigate_to(path):
            return self.navigate_to(path)

    def register_modules(self, app):
        for page in self.pages:
            if page.module is None:
                continue
            page.module.register(app)


def make_burger(pages, before_pages=None, after_pages=None, href=None, **kwargs):
    children = []
    for i, page in enumerate(pages):
        children.append(html.A(children=page.label, href=href(page) if href is not None else "/{}".format(page.id)))
        if i < (len(pages) - 1):
            children.append(html.Br())
    children = before_pages + children if before_pages is not None else children
    children = children + after_pages if after_pages is not None else children
    return Burger(children=children, **kwargs)



from .Burger import Burger
from .Download import Download
from .Keyboard import Keyboard
from .Lottie import Lottie
from .Monitor import Monitor

__all__ = [
    "Burger",
    "Download",
    "Keyboard",
    "Lottie",
    "Monitor"
]


# AUTO GENERATED FILE - DO NOT EDIT

from dash.development.base_component import Component, _explicitize_args


class Linker(Component):
    """A Linker component.
The Sync component makes it possible to synchronize states between components.

Keyword arguments:
- children (list of a list of or a singular dash component, string or numbers; optional): The children of this component. Must be a list of components with length > 1.
- links (dict; optional): List of links. Each link is a list of dicts that specify which properties to synchronize. links has the following type: list of list of dicts containing keys 'id', 'prop'.
Those keys have the following types:
  - id (string; required)
  - prop (boolean | number | string | dict | list; required)s
- id (string; optional): The ID used to identify this component in Dash callbacks.
- style (dict; optional): The CSS style of the component.
- className (string; optional): A custom class name."""
    @_explicitize_args
    def __init__(self, children=None, links=Component.UNDEFINED, id=Component.UNDEFINED, style=Component.UNDEFINED, className=Component.UNDEFINED, **kwargs):
        self._prop_names = ['children', 'links', 'id', 'style', 'className']
        self._type = 'Linker'
        self._namespace = 'dash_extensions'
        self._valid_wildcard_attributes =            []
        self.available_properties = ['children', 'links', 'id', 'style', 'className']
        self.available_wildcard_properties =            []

        _explicit_args = kwargs.pop('_explicit_args')
        _locals = locals()
        _locals.update(kwargs)  # For wildcard attrs
        args = {k: _locals[k] for k in _explicit_args if k != 'children'}

        for k in []:
            if k not in args:
                raise TypeError(
                    'Required argument `' + k + '` was not specified.')
        super(Linker, self).__init__(children=children, **args)
